@@ -45,13 +45,6 @@ extension AlbumModel {
     }
 }
 
-// MARK: - Edit
-extension AlbumModel: EditableMLModel {
-    func editCellType() -> BaseCollectionViewCell.Type {
-        return MediaEditCell.self
-    }
-}
-
 // MARK: - Search
 extension VLCMLAlbum: SearchableMLModel {
     func contains(_ searchString: String) -> Bool {
@@ -105,6 +98,10 @@ extension AlbumModel: MediaLibraryObserver {
         }
         updateView?()
     }
+
+    func medialibraryDidStartRescan() {
+        files.removeAll()
+    }
 }
 
 extension VLCMLAlbum: MediaCollectionModel {
@@ -112,18 +109,14 @@ extension VLCMLAlbum: MediaCollectionModel {
         return nil
     }
 
-    func files() -> [VLCMLMedia]? {
-        return tracks
+    func files(with criteria: VLCMLSortingCriteria = .alpha,
+               desc: Bool = false) -> [VLCMLMedia]? {
+        return tracks(with: criteria, desc: desc)
     }
 
     func title() -> String {
         return title
     }
-
-    func sortFilesInCollection(with criteria: VLCMLSortingCriteria,
-                               desc: Bool) -> [VLCMLMedia]? {
-        return nil
-    }
 }
 
 extension VLCMLAlbum {
@@ -134,21 +127,6 @@ extension VLCMLAlbum {
         return String(format: tracksString, trackCount)
     }
 
-    @objc func thumbnail() -> UIImage? {
-        var image = UIImage(contentsOfFile: artworkMRL()?.path ?? "")
-        if image == nil {
-            for track in files() ?? [] where track.isThumbnailGenerated() {
-                image = UIImage(contentsOfFile: track.thumbnail()?.path ?? "")
-                break
-            }
-        }
-        if image == nil {
-            let isDarktheme = PresentationTheme.current == PresentationTheme.darkTheme
-            image = isDarktheme ? UIImage(named: "album-placeholder-dark") : UIImage(named: "album-placeholder-white")
-        }
-        return image
-    }
-
     func albumName() -> String {
         return isUnknownAlbum() ? NSLocalizedString("UNKNOWN_ALBUM", comment: "") : title
     }
