@@ -45,13 +45,6 @@ extension ArtistModel {
     }
 }
 
-// MARK: - Edit
-extension ArtistModel: EditableMLModel {
-    func editCellType() -> BaseCollectionViewCell.Type {
-        return MediaEditCell.self
-    }
-}
-
 // MARK: - Search
 extension VLCMLArtist: SearchableMLModel {
     func contains(_ searchString: String) -> Bool {
@@ -90,6 +83,9 @@ extension ArtistModel: MediaLibraryObserver {
         updateView?()
     }
 
+    func medialibraryDidStartRescan() {
+        files.removeAll()
+    }
 }
 
 extension VLCMLArtist: MediaCollectionModel {
@@ -98,8 +94,9 @@ extension VLCMLArtist: MediaCollectionModel {
         return SortModel([.alpha])
     }
 
-    func files() -> [VLCMLMedia]? {
-        return tracks()
+    func files(with criteria: VLCMLSortingCriteria = .alpha,
+               desc: Bool = false) -> [VLCMLMedia]? {
+        return tracks(with: criteria, desc: desc)
     }
 
     func title() -> String {
@@ -118,21 +115,6 @@ extension VLCMLArtist {
         return String(format: tracksString, tracks()?.count ?? 0)
     }
 
-    @objc func thumbnail() -> UIImage? {
-        var image = UIImage(contentsOfFile: artworkMRL()?.path ?? "")
-        if image == nil {
-            for track in files() ?? [] where track.isThumbnailGenerated() {
-                image = UIImage(contentsOfFile: track.thumbnail()?.path ?? "")
-                break
-            }
-        }
-        if image == nil {
-            let isDarktheme = PresentationTheme.current == PresentationTheme.darkTheme
-            image = isDarktheme ? UIImage(named: "artist-placeholder-dark") : UIImage(named: "artist-placeholder-white")
-        }
-        return image
-    }
-
     func artistName() -> String {
         if identifier() == UnknownArtistID {
             return NSLocalizedString("UNKNOWN_ARTIST", comment: "")
