@@ -62,24 +62,18 @@ class CollectionModel: MLBaseModel {
             catch let error as NSError {
                 assertionFailure("CollectionModel: Delete failed: \(error.localizedDescription)")
             }
+            filterFilesFromDeletion(of: items)
         }
     }
 
     func sort(by criteria: VLCMLSortingCriteria, desc: Bool) {
-        files = mediaCollection.sortFilesInCollection(with: criteria, desc: desc) ?? []
+        files = mediaCollection.files(with: criteria, desc: desc) ?? []
         sortModel.currentSort = criteria
         sortModel.desc = desc
         updateView?()
     }
 }
 
-// MARK: - Edit
-extension CollectionModel: EditableMLModel {
-    func editCellType() -> BaseCollectionViewCell.Type {
-        return MediaEditCell.self
-    }
-}
-
 // MARK: - MediaLibraryObserver
 extension CollectionModel: MediaLibraryObserver {
     func medialibrary(_ medialibrary: MediaLibraryService,
@@ -91,6 +85,7 @@ extension CollectionModel: MediaLibraryObserver {
     }
 
     func medialibrary(_ medialibrary: MediaLibraryService, didModifyTracks tracks: [VLCMLMedia]) {
+        files = mediaCollection.files() ?? []
         updateView?()
     }
 
@@ -98,5 +93,9 @@ extension CollectionModel: MediaLibraryObserver {
         files = mediaCollection.files() ?? []
         updateView?()
     }
+
+    func medialibraryDidStartRescan() {
+        files.removeAll()
+    }
 }
 
