@@ -65,6 +65,9 @@ extension GenreModel: MediaLibraryObserver {
         updateView?()
     }
 
+    func medialibraryDidStartRescan() {
+        files.removeAll()
+    }
 }
 
 // MARK: - Sort
@@ -76,12 +79,6 @@ extension GenreModel {
         updateView?()
     }
 }
-// MARK: - Edit
-extension GenreModel: EditableMLModel {
-    func editCellType() -> BaseCollectionViewCell.Type {
-        return MediaEditCell.self
-    }
-}
 
 // MARK: - Search
 extension VLCMLGenre: SearchableMLModel {
@@ -100,19 +97,6 @@ extension VLCMLGenre {
         return String(format: NSLocalizedString("TRACK", comment: ""), numberOftracks)
     }
 
-    func thumbnail() -> UIImage? {
-        var image: UIImage? = nil
-        for track in tracks() ?? [] where track.isThumbnailGenerated() {
-            image = UIImage(contentsOfFile: track.thumbnail()?.path ?? "")
-            break
-        }
-        if image == nil {
-            let isDarktheme = PresentationTheme.current == PresentationTheme.darkTheme
-            image = isDarktheme ? UIImage(named: "song-placeholder-dark") : UIImage(named: "song-placeholder-white")
-        }
-        return image
-    }
-
     func accessibilityText() -> String? {
         return name + " " + numberOfTracksString()
     }
@@ -123,16 +107,12 @@ extension VLCMLGenre: MediaCollectionModel {
         return SortModel([.alpha])
     }
 
-    func files() -> [VLCMLMedia]? {
-        return tracks()
+    func files(with criteria: VLCMLSortingCriteria = .alpha,
+               desc: Bool = false) -> [VLCMLMedia]? {
+        return tracks(with: criteria, desc: desc)
     }
 
     func title() -> String {
         return name
     }
-
-    func sortFilesInCollection(with criteria: VLCMLSortingCriteria,
-                               desc: Bool) -> [VLCMLMedia]? {
-        return nil
-    }
 }
