@@ -64,20 +64,15 @@ extension MLBaseModel {
     }
 }
 
-protocol EditableMLModel {
-    func editCellType() -> BaseCollectionViewCell.Type
-}
-
 protocol SearchableMLModel {
     func contains(_ searchString: String) -> Bool
 }
 
 protocol MediaCollectionModel {
-    func files() -> [VLCMLMedia]?
+    func files(with criteria: VLCMLSortingCriteria,
+               desc: Bool) -> [VLCMLMedia]?
     func sortModel() -> SortModel?
     func title() -> String
-    func sortFilesInCollection(with criteria: VLCMLSortingCriteria,
-                               desc: Bool) -> [VLCMLMedia]?
 }
 
 // MARK: - Helper methods
@@ -100,6 +95,15 @@ extension MLBaseModel {
         }
         return newFiles
     }
+
+    func filterFilesFromDeletion(of items: [VLCMLObject]) {
+        files = files.filter() {
+            for item in items where $0.identifier() == item.identifier() {
+                return false
+            }
+            return true
+        }
+    }
 }
 
 extension VLCMLObject {
@@ -107,3 +111,31 @@ extension VLCMLObject {
         return lhs.identifier() == rhs.identifier()
     }
 }
+
+extension MediaCollectionModel {
+    func files(with criteria: VLCMLSortingCriteria = .alpha,
+               desc: Bool = false) -> [VLCMLMedia]? {
+        return files(with: criteria, desc: desc)
+    }
+
+    func thumbnail() -> UIImage? {
+        var image: UIImage? = nil
+        if image == nil {
+            for track in files() ?? [] where track.isThumbnailGenerated() {
+                image = UIImage(contentsOfFile: track.thumbnail()?.path ?? "")
+                break
+            }
+        }
+        if image == nil
+            || (!UserDefaults.standard.bool(forKey: kVLCSettingShowThumbnails) && self is VLCMLVideoGroup)
+            || (!UserDefaults.standard.bool(forKey: kVLCSettingShowArtworks) && !(self is VLCMLVideoGroup)) {
+            let isDarktheme = PresentationTheme.current == PresentationTheme.darkTheme
+            if self is VLCMLVideoGroup {
+                image = isDarktheme ? UIImage(named: "movie-placeholder-dark") : UIImage(named: "movie-placeholder-white")
+            } else {
+                image = isDarktheme ? UIImage(named: "album-placeholder-dark") : UIImage(named: "album-placeholder-white")
+            }
+        }
+        return image
+    }
+}
