@@ -40,7 +40,8 @@ extension NSNotification {
 
     // Tumbnail
     @objc optional func medialibrary(_ medialibrary: MediaLibraryService,
-                                     thumbnailReady media: VLCMLMedia)
+                                     thumbnailReady media: VLCMLMedia,
+                                     type: VLCMLThumbnailSizeType, success: Bool)
 
     // Tracks
     @objc optional func medialibrary(_ medialibrary: MediaLibraryService,
@@ -92,6 +93,9 @@ extension NSNotification {
 
     @objc optional func medialibrary(_ medialibrary: MediaLibraryService,
                                      didDeletePlaylistsWithIds playlistsIds: [NSNumber])
+
+    // Force Rescan
+    @objc optional func medialibraryDidStartRescan()
 }
 
 // MARK: -
@@ -369,7 +373,6 @@ extension MediaLibraryService {
     }
 
     func savePlaybackState(from player: PlaybackService) {
-
         let media: VLCMedia? = player.currentlyPlayingMedia
         guard let mlMedia = fetchMedia(with: media?.url.absoluteURL) else {
             // we opened a url and not a local file
@@ -384,8 +387,12 @@ extension MediaLibraryService {
         mlMedia.titleIndex = Int64(player.indexOfCurrentTitle)
         //create a new thumbnail
     }
+}
 
-    private func handleWillEnterForegroundNotification() {
+// MARK: - Application notifications
+
+@objc private extension MediaLibraryService {
+    @objc private func handleWillEnterForegroundNotification() {
         guard let documentPath = NSSearchPathForDirectoriesInDomains(.documentDirectory, .userDomainMask, true).first else {
             assertionFailure("MediaLibraryService: handleWillEnterForegroundNotification: Failed to retrieve documentPath")
             return
@@ -540,9 +547,11 @@ extension MediaLibraryService: VLCMediaLibraryDelegate {
         }
     }
 
-    func medialibrary(_ medialibrary: VLCMediaLibrary, thumbnailReadyFor media: VLCMLMedia, withSuccess success: Bool) {
+    func medialibrary(_ medialibrary: VLCMediaLibrary, thumbnailReadyFor media: VLCMLMedia,
+                      of type: VLCMLThumbnailSizeType, withSuccess success: Bool) {
         for observer in observers {
-            observer.value.observer?.medialibrary?(self, thumbnailReady: media)
+            observer.value.observer?.medialibrary?(self, thumbnailReady: media,
+                                                   type: type, success: success)
         }
     }
 }
@@ -594,6 +603,7 @@ extension MediaLibraryService {
 }
 
 // MARK: - VLCMediaLibraryDelegate - Genres
+
 extension MediaLibraryService {
     func medialibrary(_ medialibrary: VLCMediaLibrary, didAdd genres: [VLCMLGenre]) {
         for observer in observers {
@@ -658,3 +668,28 @@ extension MediaLibraryService {
         }
     }
 }
+
+// MARK: - VLCMediaLibraryDelegate - Exception handling
+
+extension MediaLibraryService {
+    func medialibrary(_ medialibrary: VLCMediaLibrary,
+                      unhandledExceptionWithContext context: String,
+                      errorMessage: String, clearSuggested: Bool) -> Bool {
+        if clearSuggested {
+            medialib.clearDatabase(restorePlaylists: true)
+            setupMediaLibrary()
+            return true
+        }
+        return false
+    }
+}
+
+// MARK: - VLCMLMediaLibraryDelegate - Force rescan
+
+extension MediaLibraryService {
+    func medialibraryDidStartRescan(_ medialibrary: VLCMediaLibrary) {
+        for observer in observers {
+            observer.value.observer?.medialibraryDidStartRescan?()
+        }
+    }
+}
