@@ -13,44 +13,31 @@ protocol EditToolbarDelegate: class {
     func editToolbarDidDelete(_ editToolbar: EditToolbar)
     func editToolbarDidAddToPlaylist(_ editToolbar: EditToolbar)
     func editToolbarDidRename(_ editToolbar: EditToolbar)
-    func editToolbarDidShare(_ editToolbar: EditToolbar, presentFrom button: UIButton)
+    func editToolbarDidShare(_ editToolbar: EditToolbar)
 }
 
 class EditToolbar: UIView {
     static let height: CGFloat = 60
     weak var delegate: EditToolbarDelegate?
-    private var category: MediaLibraryBaseModel
-    private var stackView = UIStackView()
-    private var shareButton: UIButton = {
-        let shareButton = UIButton(type: .system)
-        shareButton.addTarget(self, action: #selector(share), for: .touchUpInside)
-        shareButton.setImage(UIImage(named: "share"), for: .normal)
-        shareButton.tintColor = .orange
-        shareButton.widthAnchor.constraint(equalToConstant: 44).isActive = true
-        shareButton.accessibilityLabel = NSLocalizedString("SHARE_LABEL", comment: "")
-        shareButton.accessibilityHint = NSLocalizedString("SHARE_HINT", comment: "")
-        return shareButton
+    private var model: MediaLibraryBaseModel
+
+    private var stackView: UIStackView = {
+        let stackView = UIStackView()
+
+        stackView.translatesAutoresizingMaskIntoConstraints = false
+        stackView.distribution = .equalSpacing
+
+        return stackView
     }()
-    private var renameButton: UIButton = {
-        let renameButton = UIButton(type: .system)
-        renameButton.addTarget(self, action: #selector(rename), for: .touchUpInside)
-        renameButton.setImage(UIImage(named: "rename"), for: .normal)
-        renameButton.tintColor = .orange
-        renameButton.widthAnchor.constraint(equalToConstant: 44).isActive = true
-        renameButton.accessibilityLabel = NSLocalizedString("BUTTON_RENAME", comment: "")
-        renameButton.accessibilityHint = NSLocalizedString("RENAME_HINT", comment: "")
-        return renameButton
-    }()
-    private var deleteButton: UIButton = {
-        let deleteButton = UIButton(type: .system)
-        deleteButton.addTarget(self, action: #selector(deleteSelection), for: .touchUpInside)
-        deleteButton.setImage(UIImage(named: "delete"), for: .normal)
-        deleteButton.tintColor = .orange
-        deleteButton.widthAnchor.constraint(equalToConstant: 44).isActive = true
-        deleteButton.accessibilityLabel = NSLocalizedString("BUTTON_DELETE", comment: "")
-        deleteButton.accessibilityHint = NSLocalizedString("DELETE_HINT", comment: "")
-        return deleteButton
+
+    private var rightStackView: UIStackView = {
+        let rightStackView = UIStackView()
+
+        rightStackView.translatesAutoresizingMaskIntoConstraints = false
+
+        return rightStackView
     }()
+
     private var addToPlaylistButton: UIButton = {
         let addToPlaylistButton = UIButton(type: .system)
         addToPlaylistButton.setTitle(NSLocalizedString("ADD_TO_PLAYLIST", comment: ""), for: .normal)
@@ -76,32 +63,43 @@ class EditToolbar: UIView {
     }
 
     @objc func share() {
-        delegate?.editToolbarDidShare(self, presentFrom: shareButton)
+        delegate?.editToolbarDidShare(self)
+    }
+
+    private func setupRightStackView() {
+        var buttonList = EditButtonsFactory.buttonList(for: model.anyfiles.first)
+        // For now we remove the first button which is Add to playlist since it is not in the same group
+        if buttonList.contains(.addToPlaylist) {
+            if let index = buttonList.firstIndex(of: .addToPlaylist) {
+                buttonList.remove(at: index)
+            }
+        }
+        let buttons = EditButtonsFactory.generate(buttons: buttonList)
+        for button in buttons {
+            switch button.identifier {
+                case .addToPlaylist:
+                    rightStackView.addArrangedSubview(button.button(#selector(addToPlaylist)))
+                case .rename:
+                    rightStackView.addArrangedSubview(button.button(#selector(rename)))
+                case .delete:
+                    rightStackView.addArrangedSubview(button.button(#selector(deleteSelection)))
+                case .share:
+                    rightStackView.addArrangedSubview(button.button(#selector(share)))
+            }
+        }
     }
 
     private func setupStackView() {
-        let stackView = UIStackView(arrangedSubviews: [addToPlaylistButton])
-        let file = category.anyfiles.first
+        stackView.addArrangedSubview(addToPlaylistButton)
+        stackView.addArrangedSubview(rightStackView)
 
-        if !(file is VLCMLArtist) && !(file is VLCMLGenre) && !(file is VLCMLAlbum)
-            && !(file is VLCMLVideoGroup) {
-            stackView.addArrangedSubview(renameButton)
-        }
-
-        if  !(file is VLCMLVideoGroup) {
-            stackView.addArrangedSubview(deleteButton)
-        }
-
-        stackView.addArrangedSubview(shareButton)
-
-        stackView.translatesAutoresizingMaskIntoConstraints = false
         addSubview(stackView)
         NSLayoutConstraint.activate([
             stackView.topAnchor.constraint(equalTo: topAnchor),
             stackView.leadingAnchor.constraint(equalTo: leadingAnchor, constant: 20),
-            stackView.trailingAnchor.constraint(equalTo: trailingAnchor, constant: -20),
+            stackView.trailingAnchor.constraint(equalTo: trailingAnchor, constant: -10),
             stackView.bottomAnchor.constraint(equalTo: bottomAnchor)
-            ])
+        ])
     }
 
     private func setupView() {
@@ -110,15 +108,15 @@ class EditToolbar: UIView {
         backgroundColor = PresentationTheme.current.colors.background
     }
 
-    init(category: MediaLibraryBaseModel) {
-        self.category = category
+    init(model: MediaLibraryBaseModel) {
+        self.model = model
         super.init(frame: .zero)
         setupView()
+        setupRightStackView()
         setupStackView()
     }
 
     required init(coder: NSCoder) {
         fatalError("init(coder:) has not been implemented")
     }
-
 }
