@@ -17,6 +17,7 @@ import Foundation
     func needsToUpdateNavigationbarIfNeeded(_ viewController: MediaCategoryViewController)
     func enableCategorySwitching(for viewController: MediaCategoryViewController,
                                  enable: Bool)
+    func setEditingStateChanged(for viewController: MediaCategoryViewController, editing: Bool)
 }
 
 class MediaCategoryViewController: UICollectionViewController, UISearchBarDelegate, IndicatorInfoProvider {
@@ -92,13 +93,6 @@ class MediaCategoryViewController: UICollectionViewController, UISearchBarDelega
         return emptyView
     }()
 
-    let editCollectionViewLayout: UICollectionViewFlowLayout = {
-        let editCollectionViewLayout = UICollectionViewFlowLayout()
-        editCollectionViewLayout.minimumLineSpacing = 1
-        editCollectionViewLayout.minimumInteritemSpacing = 0
-        return editCollectionViewLayout
-    }()
-
     @available(*, unavailable)
     init() {
         fatalError()
@@ -167,18 +161,18 @@ class MediaCategoryViewController: UICollectionViewController, UISearchBarDelega
     }
 
     @objc func reloadData() {
-        DispatchQueue.main.async {
-            [weak self] in
-            guard let self = self else {
-                return
+        guard Thread.isMainThread else {
+            DispatchQueue.main.async {
+                self.reloadData()
             }
-            self.delegate?.needsToUpdateNavigationbarIfNeeded(self)
-            self.collectionView?.reloadData()
-            self.updateUIForContent()
+            return
+        }
+        delegate?.needsToUpdateNavigationbarIfNeeded(self)
+        collectionView?.reloadData()
+        updateUIForContent()
 
-            if !self.isSearching {
-                self.popViewIfNecessary()
-            }
+        if !isSearching {
+            popViewIfNecessary()
         }
     }
 
@@ -290,6 +284,10 @@ class MediaCategoryViewController: UICollectionViewController, UISearchBarDelega
     }
 
     override func setEditing(_ editing: Bool, animated: Bool) {
+        guard editing != isEditing else {
+            // Guard in case where setEditing is called twice with the same state
+            return
+        }
         super.setEditing(editing, animated: animated)
         // might have an issue if the old datasource was search
         // Most of the edit logic is handled inside editController
@@ -301,16 +299,8 @@ class MediaCategoryViewController: UICollectionViewController, UISearchBarDelega
 
         PlaybackService.sharedInstance().setPlayerHidden(editing)
 
-        let layoutToBe = editing ? editCollectionViewLayout : UICollectionViewFlowLayout()
-        collectionView?.setCollectionViewLayout(layoutToBe, animated: false, completion: {
-            [unowned self] finished in
-            guard finished else {
-                assertionFailure("VLCMediaSubcategoryViewController: Edit layout transition failed.")
-                return
-            }
-            self.searchBarConstraint?.constant = -self.searchBarSize
-            self.reloadData()
-        })
+        searchBarConstraint?.constant = -self.searchBarSize
+        reloadData()
     }
 
     private func displayEditToolbar() {
@@ -367,6 +357,78 @@ class MediaCategoryViewController: UICollectionViewController, UISearchBarDelega
         }
     }
 
+    func objects(from modelContent: VLCMLObject) -> [VLCMLObject] {
+        if let media = modelContent as? VLCMLMedia {
+            return [media]
+        } else if let mediaCollection = modelContent as? MediaCollectionModel {
+            return mediaCollection.files() ?? [VLCMLObject]()
+        }
+        return [VLCMLObject]()
+    }
+
+    @available(iOS 13.0, *)
+    override func collectionView(_ collectionView: UICollectionView,
+                                 contextMenuConfigurationForItemAt indexPath: IndexPath,
+                                 point: CGPoint) -> UIContextMenuConfiguration? {
+        let cell = collectionView.cellForItem(at: indexPath)
+        var thumbnail: UIImage? = nil
+        if let cell = cell as? MovieCollectionViewCell {
+            thumbnail = cell.thumbnailView.image
+        } else if let cell = cell as? MediaCollectionViewCell {
+            thumbnail = cell.thumbnailView.image
+        }
+        let configuration = UIContextMenuConfiguration(identifier: nil, previewProvider: {
+            if let thumbnail = thumbnail {
+                return CollectionViewCellPreviewController(thumbnail: thumbnail)
+            } else {
+                return nil
+            }
+        }) {
+                [weak self] action in
+                let modelContent = self?.isSearching ?? false ? self?.searchDataSource.objectAtIndex(index: indexPath.row) : self?.model.anyfiles[indexPath.row]
+
+                let actionList = EditButtonsFactory.buttonList(for: self?.model.anyfiles.first)
+                let actions = EditButtonsFactory.generate(buttons: actionList)
+                return UIMenu(title: "", image: nil, identifier: nil, children: actions.map {
+                    switch $0.identifier {
+                    case .addToPlaylist:
+                        return $0.action({
+                            [weak self] _ in
+                            if let modelContent = modelContent {
+                                self?.editController.editActions.objects = self?.objects(from: modelContent) ?? []
+                                self?.editController.editActions.addToPlaylist()
+                            }
+                        })
+                    case .rename:
+                        return $0.action({
+                            [weak self] _ in
+                            if let modelContent = modelContent {
+                                self?.editController.editActions.objects = [modelContent]
+                                self?.editController.editActions.rename()
+                            }
+                        })
+                    case .delete:
+                        return $0.action({
+                            [weak self] _ in
+                            if let modelContent = modelContent {
+                                self?.editController.editActions.objects = [modelContent]
+                                self?.editController.editActions.delete()
+                            }
+                        })
+                    case .share:
+                        return $0.action({
+                            [weak self] _ in
+                            if let modelContent = modelContent {
+                                self?.editController.editActions.objects = self?.objects(from: modelContent) ?? []
+                                self?.editController.editActions.share()
+                            }
+                        })
+                    }
+                })
+            }
+            return configuration
+    }
+
     func createSpotlightItem(media: VLCMLMedia) {
         if KeychainCoordinator.passcodeLockEnabled {
             return
@@ -403,7 +465,7 @@ extension MediaCategoryViewController {
                              for: .touchUpInside)
         sortButton
             .addGestureRecognizer(UILongPressGestureRecognizer(target: self,
-                                                               action: #selector(handleSortShortcut)))
+                                                               action: #selector(handleSortLongPress(sender:))))
 
         sortButton.tintColor = PresentationTheme.current.colors.orangeUI
         sortButton.accessibilityLabel = NSLocalizedString("BUTTON_SORT", comment: "")
@@ -415,8 +477,8 @@ extension MediaCategoryViewController {
         var rightBarButtonItems = [UIBarButtonItem]()
 
         rightBarButtonItems.append(editBarButton)
-        // Sort is only available for VideoGroups
-        if let model = model as? CollectionModel, model.mediaCollection is VLCMLVideoGroup {
+        // Sort is not available for Playlists
+        if let model = model as? CollectionModel, !(model.mediaCollection is VLCMLPlaylist) {
             rightBarButtonItems.append(sortBarButton)
         }
         rightBarButtonItems.append(rendererBarButton)
@@ -439,13 +501,21 @@ extension MediaCategoryViewController {
         }
     }
 
+    @objc func handleSortLongPress(sender: UILongPressGestureRecognizer) {
+        if sender.state == .began {
+            if #available(iOS 10.0, *) {
+                UIImpactFeedbackGenerator(style: .medium).impactOccurred()
+            }
+            handleSortShortcut()
+        }
+    }
+
     @objc func handleSortShortcut() {
         model.sort(by: model.sortModel.currentSort, desc: !model.sortModel.desc)
     }
 
     @objc func handleEditing() {
         isEditing = !isEditing
-        setEditing(isEditing, animated: true)
         navigationItem.rightBarButtonItems = isEditing ? [UIBarButtonItem(barButtonSystemItem: .done,
                                                                           target: self,
                                                                           action: #selector(handleEditing))]
@@ -583,8 +653,8 @@ extension MediaCategoryViewController: ActionSheetSortSectionHeaderDelegate {
 // MARK: - EditControllerDelegate
 
 extension MediaCategoryViewController: EditControllerDelegate {
-    func editController(editController: EditController, cellforItemAt indexPath: IndexPath) -> MediaEditCell? {
-        return collectionView.cellForItem(at: indexPath) as? MediaEditCell
+    func editController(editController: EditController, cellforItemAt indexPath: IndexPath) -> BaseCollectionViewCell? {
+        return collectionView.cellForItem(at: indexPath) as? BaseCollectionViewCell
     }
 
     func editController(editController: EditController,
@@ -592,16 +662,21 @@ extension MediaCategoryViewController: EditControllerDelegate {
         let newNavigationController = UINavigationController(rootViewController: viewController)
         navigationController?.present(newNavigationController, animated: true, completion: nil)
     }
+
+    func editControllerDidFinishEditing(editController: EditController?) {
+        // NavigationItems for Collections are create from the parent, there is no need to propagate the information.
+        if self is CollectionCategoryViewController {
+            handleEditing()
+        } else {
+            delegate?.setEditingStateChanged(for: self, editing: false)
+        }
+    }
 }
 
 private extension MediaCategoryViewController {
     func setupCollectionView() {
         let cellNib = UINib(nibName: model.cellType.nibName, bundle: nil)
         collectionView?.register(cellNib, forCellWithReuseIdentifier: model.cellType.defaultReuseIdentifier)
-        if let editCell = (model as? EditableMLModel)?.editCellType() {
-            let editCellNib = UINib(nibName: editCell.nibName, bundle: nil)
-            collectionView?.register(editCellNib, forCellWithReuseIdentifier: editCell.defaultReuseIdentifier)
-        }
         collectionView.allowsMultipleSelection = true
         collectionView?.backgroundColor = PresentationTheme.current.colors.background
         collectionView?.alwaysBounceVertical = true
