@@ -14,12 +14,16 @@ import Foundation
 
 class MediaCollectionViewCell: BaseCollectionViewCell {
 
-    @IBOutlet private weak var thumbnailView: UIImageView!
+    @IBOutlet weak var thumbnailView: UIImageView!
     @IBOutlet private weak var titleLabel: UILabel!
     @IBOutlet private weak var descriptionLabel: UILabel!
     @IBOutlet private weak var newLabel: UILabel!
     @IBOutlet private weak var thumbnailWidth: NSLayoutConstraint!
 
+    @IBOutlet weak var checkboxImageView: UIImageView!
+    @IBOutlet weak var selectionOverlay: UIView!
+    @IBOutlet weak var dragIndicatorImageView: UIImageView!
+
     override var media: VLCMLObject? {
         didSet {
             if let media = media as? VLCMLMedia {
@@ -42,6 +46,21 @@ class MediaCollectionViewCell: BaseCollectionViewCell {
         }
     }
 
+    override var checkImageView: UIImageView? {
+        return checkboxImageView
+    }
+
+    override var selectionViewOverlay: UIView? {
+        return selectionOverlay
+    }
+
+    override var isSelected: Bool {
+        didSet {
+            checkboxImageView.image = isSelected ? UIImage(named: "checkboxSelected")
+                : UIImage(named: "checkboxEmpty")
+        }
+    }
+
     override func awakeFromNib() {
         super.awakeFromNib()
         if #available(iOS 11.0, *) {
@@ -59,14 +78,11 @@ class MediaCollectionViewCell: BaseCollectionViewCell {
         backgroundColor = PresentationTheme.current.colors.background
         titleLabel?.textColor = PresentationTheme.current.colors.cellTextColor
         descriptionLabel?.textColor = PresentationTheme.current.colors.cellDetailTextColor
+        dragIndicatorImageView.tintColor = PresentationTheme.current.colors.cellDetailTextColor
     }
 
     func update(audiotrack: VLCMLMedia) {
-        var title = audiotrack.title
-        if  UserDefaults.standard.bool(forKey: kVLCOptimizeItemNamesForDisplay) == true {
-            title = (title as NSString).deletingPathExtension
-        }
-        titleLabel.text = title
+        titleLabel.text = audiotrack.title()
         accessibilityLabel = audiotrack.accessibilityText(editing: false)
         descriptionLabel.text = audiotrack.albumTrackArtistName()
         newLabel.isHidden = !audiotrack.isNew
@@ -74,6 +90,7 @@ class MediaCollectionViewCell: BaseCollectionViewCell {
     }
 
     func update(album: VLCMLAlbum) {
+        newLabel.isHidden = true
         titleLabel.text = album.albumName()
         accessibilityLabel = album.accessibilityText(editing: false)
         descriptionLabel.text = album.albumArtistName()
@@ -81,6 +98,7 @@ class MediaCollectionViewCell: BaseCollectionViewCell {
     }
 
     func update(artist: VLCMLArtist) {
+        newLabel.isHidden = true
         thumbnailView.layer.masksToBounds = true
         thumbnailView.layer.cornerRadius = thumbnailView.frame.size.width / 2.0
         titleLabel.text = artist.artistName()
@@ -90,11 +108,7 @@ class MediaCollectionViewCell: BaseCollectionViewCell {
     }
 
     func update(movie: VLCMLMedia) {
-        var title = movie.title
-        if  UserDefaults.standard.bool(forKey: kVLCOptimizeItemNamesForDisplay) == true {
-            title = (title as NSString).deletingPathExtension
-        }
-        titleLabel.text = title
+        titleLabel.text = movie.title()
         accessibilityLabel = movie.accessibilityText(editing: false)
         descriptionLabel.text = movie.mediaDuration()
         thumbnailView.image = movie.thumbnailImage()
@@ -151,5 +165,8 @@ class MediaCollectionViewCell: BaseCollectionViewCell {
         descriptionLabel.text = ""
         thumbnailView.image = nil
         newLabel.isHidden = true
+        checkboxImageView.isHidden = true
+        selectionOverlay.isHidden = true
+        dragIndicatorImageView.isHidden = true
     }
 }
